@@ -655,3 +655,25 @@ class jac_tests(TestCaseHelper, TestCase):
         test_walker.run()
         rep = test_walker.report
         self.assertTrue(rep[0].startswith('urn:uuid'))
+
+    def test_std_get_report(self):
+        gph = graph(m_id='anon', h=mem_hook())
+        sent = sentinel(m_id='anon', h=gph._h)
+        sent.register_code(jtc.std_get_report)
+        test_walker = \
+            sent.walker_ids.get_obj_by_name('init')
+        test_walker.prime(gph)
+        test_walker.run()
+        rep = test_walker.report
+        self.assertEqual(rep, [3, 5, 6, 7, [3, 5, 6, 7], 8])
+
+    def test_func_with_array_index(self):
+        gph = graph(m_id='anon', h=mem_hook())
+        sent = sentinel(m_id='anon', h=gph._h)
+        sent.register_code(jtc.func_with_array_index)
+        test_walker = \
+            sent.walker_ids.get_obj_by_name('init')
+        test_walker.prime(gph)
+        test_walker.run()
+        rep = test_walker.report
+        self.assertEqual(rep, [3, 5, 3])
